/* calsrv.cpp
 * Public domain, written by Antti P Miettinen <ananaza@iki.fi>
 */

#include <e32cons.h>
#include <c32comm.h>
#include <in_sock.h>
#include <agclient.h>

_LIT(CALENDAR_FILE,"C:\\System\\Data\\Calendar");

CConsoleBase *cons;

static void
PrintCalendarL(RSocket &s)
{
    RAgendaServ *agnServer = RAgendaServ::NewL();
    CleanupStack::PushL(agnServer);
    cons->Printf(_L("got agenda server\n"));
    agnServer->Connect();
    CleanupClosePushL(*agnServer);
    cons->Printf(_L("connected to agenda server\n"));
    RFs iFs;
    User::LeaveIfError(iFs.Connect());
    cons->Printf(_L("connected to fs\n"));
    CAgnEntryModel *model = CAgnEntryModel::NewL();
    CleanupStack::PushL(model);
    cons->Printf(_L("got model\n"));
    model->SetServer(agnServer);
    model->OpenL(CALENDAR_FILE);
    cons->Printf(_L("Opened calfile\n"));

    // Go through all entries
    TBool available = agnServer->CreateEntryIterator();
    while (available)
    {
	TAgnEntryId id = agnServer->EntryIteratorPosition();
	CAgnEntry *entry = model->FetchEntryL(id);
	if (entry->Type() != CAgnEntry::EAppt)
	{
	    cons->Printf(_L("Not appointment\n"));
	    delete entry;
	    available = agnServer->EntryIteratorNext();
	    continue;	    
	}
	TBuf<256> title;
	CRichText *content = entry->RichTextL();
	if (!content)
	{
	    cons->Printf(_L("Null content\n"));
	    delete entry;
	    available = agnServer->EntryIteratorNext();
	    continue;
	}
	TInt len = content->DocumentLength();
	if (len > 60)
	    len = 60;
	content->Extract(title,0,60);
	cons->Printf(_L("title: %S\n"), &title);
	CBufFlat *buf = CBufFlat::NewL(1024);
	RBufWriteStream ws;
	ws.Open(*buf);
	cons->Printf(_L("Created mem stream\n"));
	model->ExportVCalL(ws, entry);
	cons->Printf(_L("Exported entry\n"));
	TRequestStatus rs;
	s.Write(buf->Ptr(0),rs);
	cons->Printf(_L("wrote\n"));
	User::WaitForRequest(rs);
	cons->Printf(_L("written\n"));
	ws.Close();
	delete buf;
	available = agnServer->EntryIteratorNext();
	delete entry;
    }
    iFs.Close();
    CleanupStack::PopAndDestroy(3); // model, close session, agnServer
}

static void
OneConnectionL(void)
{
    RSocketServ ss;
    RSocket s;

    User::LeaveIfError(ss.Connect());
    cons->Printf(_L("connected to socket server\n"));
    User::LeaveIfError(s.Open(ss, KAfInet, KSockStream, KProtocolInetTcp));
    cons->Printf(_L("opened inet socket\n"));
    TInetAddr a(12345);
    s.Bind(a);
    cons->Printf(_L("bound to port 12345\n"));
    s.Listen(1);
    cons->Printf(_L("listening for one connection\n"));
    TRequestStatus rs;
    RSocket c;
    c.Open(ss);
    cons->Printf(_L("opened blank socket\n"));
    s.Accept(c, rs);
    cons->Printf(_L("accepted, waiting..\n"));
    User::WaitForRequest(rs);
    cons->Printf(_L("got connection\n"));
    PrintCalendarL(c);
    c.Close();
    s.Close();
}

static void
SpawnMainL(void)
{
    CActiveScheduler *sched;
    TInt error;

    error = StartC32();
    if (error != KErrNone && error != KErrAlreadyExists)
    {
	User::Leave(error);
    }

    sched = new (ELeave)CActiveScheduler;
    CleanupStack::PushL(sched);
    CActiveScheduler::Install(sched);

    cons = Console::NewL(_L("CAL"),
			 TSize(KDefaultConsWidth,
			       KDefaultConsHeight));
    CleanupStack::PushL(cons);

    cons->Printf(_L("cal started\n"));

    cons->Printf(_L("[press any key]\n"));
    cons->Getch();

    OneConnectionL();

    cons->Printf(_L("[press any key]\n"));
    cons->Getch();

    CleanupStack::PopAndDestroy(cons);
    CleanupStack::PopAndDestroy(sched);
}


GLDEF_C int E32Main(void)
{
    CTrapCleanup *cleanup = CTrapCleanup::New();

    TRAPD(error, SpawnMainL());
    if (error != KErrNone)
    {
	User::Panic(_L("CAL"), error);
    }

    delete cleanup;

    return 0;
}
