/* srv.cpp
 * Public domain, written by Antti P Miettinen <ananaza@iki.fi>
 */

#include <e32cons.h>
#include <c32comm.h>
#include <in_sock.h>

CConsoleBase *cons;

static void
OneConnectionL(void)
{
    RSocketServ ss;
    RSocket s;

    User::LeaveIfError(ss.Connect());
    cons->Printf(_L("connected to socket server\n"));
    User::LeaveIfError(s.Open(ss, KAfInet, KSockStream, KProtocolInetTcp));
    cons->Printf(_L("opened inet socket\n"));
    TInetAddr a(12345);
    s.Bind(a);
    cons->Printf(_L("bound to port 12345\n"));
    s.Listen(1);
    cons->Printf(_L("listening for one connection\n"));
    TRequestStatus rs;
    RSocket c;
    c.Open(ss);
    cons->Printf(_L("opened blank socket\n"));
    s.Accept(c, rs);
    cons->Printf(_L("accepted, waiting..\n"));
    User::WaitForRequest(rs);
    cons->Printf(_L("got connection\n"));
    c.Write(TPtrC8((TUint8 *)"hello\n"), rs);
    cons->Printf(_L("wrote\n"));
    User::WaitForRequest(rs);
    cons->Printf(_L("written\n"));
    c.Close();
    s.Close();
}

static void
SpawnMainL(void)
{
    CActiveScheduler *sched;
    TInt error;

    error = StartC32();
    if (error != KErrNone && error != KErrAlreadyExists)
    {
	User::Leave(error);
    }

    sched = new (ELeave)CActiveScheduler;
    CleanupStack::PushL(sched);
    CActiveScheduler::Install(sched);

    cons = Console::NewL(_L("SRV"),
			 TSize(KDefaultConsWidth,
			       KDefaultConsHeight));
    CleanupStack::PushL(cons);

    cons->Printf(_L("srv started\n"));

    cons->Printf(_L("[press any key]\n"));
    cons->Getch();

    OneConnectionL();

    cons->Printf(_L("[press any key]\n"));
    cons->Getch();

    CleanupStack::PopAndDestroy(cons);
    CleanupStack::PopAndDestroy(sched);
}


GLDEF_C int E32Main(void)
{
    CTrapCleanup *cleanup = CTrapCleanup::New();

    TRAPD(error, SpawnMainL());
    if (error != KErrNone)
    {
	User::Panic(_L("SRV"), error);
    }

    delete cleanup;

    return 0;
}
